=== modified file 'NEWS'
--- NEWS	2011-03-25 11:38:43 +0000
+++ NEWS	2011-11-08 16:29:37 +0000
@@ -39,11 +39,26 @@
       Improves performance on projects with long histories like emacs.
       (John Arbash Meinel)
 
+    - Fix support for displaying foreign revision ids.
+      (Jelmer Vernooij, #736026)
+
+    - Add hook 'controller' to BranchWSGIApp, allowing other bzr plugins
+      to provide custom branch-specific controllers. (Jelmer Vernooij, #585822)
+
+    - Add privacy notification code to loggerhead, allowing branches to be
+      marked as private and a notification ribbon to be displayed via
+      javascript on the view pages. (JC Sackett, #823471)
+
+
+1.18.1 [24Mar2011]
+------------------
+
     - Fix escaping of filenames in revision views.
       (William Grant, #740142)
 
-    - Fix support for displaying foreign revision ids.
-      (Jelmer Vernooij, #736026)
+    - Add missing import to loggerhead.trace, allowing start-loggerhead
+      to run when a log.roll config option is set.
+      (Max Kanat-Alexander, #673999)
 
 
 1.18 [10Nov2010]

=== modified file 'loggerhead/__init__.py'
--- loggerhead/__init__.py	2010-11-11 03:30:43 +0000
+++ loggerhead/__init__.py	2011-03-24 03:40:33 +0000
@@ -22,7 +22,7 @@
 
 import pkg_resources
 
-__version__ = '1.18'
+__version__ = '1.18.1'
 required_bzrlib = (1, 17)
 
 pkg_resources.get_distribution('Paste>=1.6')

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2011-06-28 16:09:37 +0000
+++ loggerhead/apps/branch.py	2011-11-04 18:16:45 +0000
@@ -22,6 +22,7 @@
 
 import bzrlib.branch
 import bzrlib.errors
+from bzrlib.hooks import Hooks
 import bzrlib.lru_cache
 
 from paste import request
@@ -49,7 +50,7 @@
 
     def __init__(self, branch, friendly_name=None, config={},
                  graph_cache=None, branch_link=None, is_root=False,
-                 served_url=_DEFAULT, use_cdn=False):
+                 served_url=_DEFAULT, use_cdn=False, private=False):
         self.branch = branch
         self._config = config
         self.friendly_name = friendly_name
@@ -61,25 +62,29 @@
         self.is_root = is_root
         self.served_url = served_url
         self.use_cdn = use_cdn
+        self.private = private
+
+    def public_private_css(self):
+        if self.private:
+            return "private"
+        else:
+            return "public"
 
     def get_history(self):
-        file_cache = None
         revinfo_disk_cache = None
         cache_path = self._config.get('cachepath', None)
         if cache_path is not None:
             # Only import the cache if we're going to use it.
             # This makes sqlite optional
             try:
-                from loggerhead.changecache import (
-                    FileChangeCache, RevInfoDiskCache)
+                from loggerhead.changecache import RevInfoDiskCache
             except ImportError:
                 self.log.debug("Couldn't load python-sqlite,"
                                " continuing without using a cache")
             else:
-                file_cache = FileChangeCache(cache_path)
                 revinfo_disk_cache = RevInfoDiskCache(cache_path)
         return History(
-            self.branch, self.graph_cache, file_cache=file_cache,
+            self.branch, self.graph_cache,
             revinfo_disk_cache=revinfo_disk_cache, cache_key=self.friendly_name)
 
     def url(self, *args, **kw):
@@ -158,6 +163,10 @@
                     self.served_url = self.url([])
                 except bzrlib.errors.InvalidURL:
                     self.served_url = None
+        for hook in self.hooks['controller']:
+            controller = hook(self, environ)
+            if controller is not None:
+                return controller
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(
@@ -168,9 +177,9 @@
             environ['loggerhead.as_json'] = True
             path = request.path_info_pop(environ)
         cls = self.controllers_dict.get(path)
-        if cls is None:
-            raise httpexceptions.HTTPNotFound()
-        return cls(self, self.get_history)
+        if cls is not None:
+            return cls(self, self.get_history)
+        raise httpexceptions.HTTPNotFound()
 
     def app(self, environ, start_response):
         self.branch.lock_read()
@@ -184,3 +193,23 @@
                 raise
         finally:
             self.branch.unlock()
+
+
+class BranchWSGIAppHooks(Hooks):
+    """A dictionary mapping hook name to a list of callables for WSGI app branch hooks.
+    """
+
+    def __init__(self):
+        """Create the default hooks.
+        """
+        Hooks.__init__(self, "bzrlib.plugins.loggerhead.apps.branch",
+            "BranchWSGIApp.hooks")
+        self.add_hook('controller',
+            "Invoked when looking for the controller to use for a "
+            "branch subpage. The api signature is (branch_app, environ)."
+            "If a hook can provide a controller, it should return one, "
+            "as a standard WSGI app. If it can't provide a controller, "
+            "it should return None", (1, 19))
+
+
+BranchWSGIApp.hooks = BranchWSGIAppHooks()

=== modified file 'loggerhead/changecache.py'
--- loggerhead/changecache.py	2009-05-02 14:01:05 +0000
+++ loggerhead/changecache.py	2011-08-17 11:04:00 +0000
@@ -99,24 +99,6 @@
             pass
 
 
-class FileChangeCache(object):
-
-    def __init__(self, cache_path):
-
-        if not os.path.exists(cache_path):
-            os.mkdir(cache_path)
-
-        self._changes_filename = os.path.join(cache_path, 'filechanges.sql')
-
-    def get_file_changes(self, entry):
-        cache = FakeShelf(self._changes_filename)
-        changes = cache.get(entry.revid)
-        if changes is None:
-            changes = self.history.get_file_changes_uncached(entry)
-            cache.add(entry.revid, changes)
-        return changes
-
-
 class RevInfoDiskCache(object):
     """Like `RevInfoMemoryCache` but backed in a sqlite DB."""
 

=== modified file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	2011-08-08 20:58:09 +0000
+++ loggerhead/controllers/annotate_ui.py	2011-08-09 14:06:49 +0000
@@ -46,7 +46,11 @@
                     change = self._history.get_changes([line_revid])[0]
                     change_cache[line_revid] = change
 
-                message = change.comment.splitlines()[0]
+                try:
+                    message = change.comment.splitlines()[0]
+                except IndexError:
+                    # Comment not present for this revision
+                    message = ""
 
                 if last_lineno:
                     # The revspan is of lines between the last revision and this one.

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2011-06-27 17:11:24 +0000
+++ loggerhead/controllers/revision_ui.py	2011-09-02 09:43:57 +0000
@@ -118,10 +118,14 @@
         link_data = {}
         path_to_id = {}
         if path:
-            item = [x for x in file_changes.text_changes if x.filename == path][0]
-            diff_chunks = diff_chunks_for_file(
-                self._history._branch.repository, item.file_id,
-                item.old_revision, item.new_revision)
+            items = [x for x in file_changes.text_changes if x.filename == path]
+            if len(items) > 0:
+                item = items[0]
+                diff_chunks = diff_chunks_for_file(
+                    self._history._branch.repository, item.file_id,
+                    item.old_revision, item.new_revision)
+            else:
+                diff_chunks = None
         else:
             diff_chunks = None
             for i, item in enumerate(file_changes.text_changes):

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2011-03-25 13:09:10 +0000
+++ loggerhead/history.py	2011-08-17 11:04:00 +0000
@@ -215,8 +215,6 @@
     around it, serve the request, throw the History object away, unlock the
     branch and throw it away.
 
-    :ivar _file_change_cache: An object that caches information about the
-        files that changed between two revisions.
     :ivar _rev_info: A list of information about revisions.  This is by far
         the most cryptic data structure in loggerhead.  At the top level, it
         is a list of 3-tuples [(merge-info, where-merged, parents)].
@@ -284,15 +282,10 @@
                 self._rev_indices[revid] = seq
                 self._revno_revid[revno_str] = revid
 
-    def __init__(self, branch, whole_history_data_cache, file_cache=None,
+    def __init__(self, branch, whole_history_data_cache,
                  revinfo_disk_cache=None, cache_key=None):
         assert branch.is_locked(), (
             "Can only construct a History object with a read-locked branch.")
-        if file_cache is not None:
-            self._file_change_cache = file_cache
-            file_cache.history = self
-        else:
-            self._file_change_cache = None
         self._branch = branch
         self._branch_tags = None
         self._inventory_cache = {}
@@ -759,19 +752,13 @@
             entry["foreign_revid"] = mapping.vcs.show_foreign_revid(foreign_revid)
         return util.Container(entry)
 
-    def get_file_changes_uncached(self, entry):
+    def get_file_changes(self, entry):
         if entry.parents:
             old_revid = entry.parents[0].revid
         else:
             old_revid = bzrlib.revision.NULL_REVISION
         return self.file_changes_for_revision_ids(old_revid, entry.revid)
 
-    def get_file_changes(self, entry):
-        if self._file_change_cache is None:
-            return self.get_file_changes_uncached(entry)
-        else:
-            return self._file_change_cache.get_file_changes(entry)
-
     def add_changes(self, entry):
         changes = self.get_file_changes(entry)
         entry.changes = changes

=== modified file 'loggerhead/static/css/global.css'
--- loggerhead/static/css/global.css	2011-04-22 17:22:49 +0000
+++ loggerhead/static/css/global.css	2011-10-25 15:58:27 +0000
@@ -343,3 +343,64 @@
 #footer {
     font-size: 77%;
     }
+
+    
+/* ====================
+   Global notifications
+*/
+/* Move the content down so the notification banner doesn't hide any content. */
+body.global-notification-visible {
+    padding-top: 40px;
+    }
+body.global-notification-visible .login-logout {
+    top: 45px;
+    }
+.global-notification {
+    position: fixed;
+    z-index: 10;
+    top: 0;
+    left: 0;
+    right: 0;
+    padding: 8px 20px;
+    /* Define colour for browsers that don't support transparency */
+    background-color: #8d1f1f;
+    /* Set transparent background for browsers that support it */
+    background-color: rgba(125,0,0,0.9);
+    color: #fff;
+    text-shadow: 0 -1px 0 #631616;
+    font-size: 14px;
+    line-height: 21px;
+    font-weight: bold;
+    -moz-box-shadow: 0 0 5px #333;
+    -webkit-box-shadow: 0 0 5px #333;
+    box-shadow: 0 0 5px #333;
+    }
+.global-notification .sprite.notification-private {
+    float: left;
+    display: inline-block;
+    height: 21px;
+    width: 20px;
+    margin-right: 10px;
+    padding: 0;
+    }
+.global-notification-close, .global-notification-close:active,
+.global-notification-close:visited {
+    color: #e47a7a;
+    }
+.global-notification-close {
+    display: block;
+    position: absolute;
+    top: 11px;
+    right: 20px;
+    font-size: 12px;
+    font-weight: normal;
+    line-height: 14px;
+    }
+.global-notification-close .sprite.notification-close {
+    float: right;
+    display: block;
+    height: 9px;
+    width: 8px;
+    margin: 3px 0 0 7px;
+    padding: 0;
+    }

=== modified file 'loggerhead/static/javascript/custom.js'
--- loggerhead/static/javascript/custom.js	2009-04-01 15:28:21 +0000
+++ loggerhead/static/javascript/custom.js	2011-10-26 16:06:54 +0000
@@ -1,4 +1,4 @@
-Y = YUI().use("node", "io-base", "anim");
+Y = YUI().use("base", "node", "io-base", "anim");
 
 var global_timeout_id = null;
 var global_search_request = null;
@@ -280,3 +280,89 @@
   }
 };
 
+var notification_node = null;
+/*
+ * Display privacy notifications
+ *
+ * This should be called after the page has loaded e.g. on 'domready'.
+ */
+function setup_privacy_notification(config) {
+    if (notification_node !== null) {
+        return;
+    }
+    var notification_text = 'The information on this page is private';
+    var hidden = true;
+    var target_id = "loggerheadCont";
+    if (config !== undefined) {
+        if (config.notification_text !== undefined) {
+            notification_text = config.notification_text;
+        }
+        if (config.hidden !== undefined) {
+            hidden = config.hidden;
+        }
+        if (config.target_id !== undefined) {
+            target_id = config.target_id;
+        }
+    }
+    var id_selector = "#" + target_id;
+    var main = Y.get(id_selector);
+    notification_node = Y.Node.create('<div></div>')
+        .addClass('global-notification');
+    if (hidden) {
+        notification_node.addClass('hidden');
+    }
+    var notification_span = Y.Node.create('<span></span>')
+        .addClass('sprite')
+        .addClass('notification-private');
+    notification_node.set('innerHTML', notification_text);
+    main.appendChild(notification_node);
+    notification_node.appendChild(notification_span);
+};
+
+function display_privacy_notification() {
+    /* Set a temporary class on the body for the feature flag,
+     this is because we have no way to use feature flags in
+     css directly. This should be removed if the feature
+     is accepted. */
+    var body = Y.get('body');
+    body.addClass('feature-flag-bugs-private-notification-enabled');
+    // Set the visible flag so that the content moves down.
+    body.addClass('global-notification-visible');
+
+    setup_privacy_notification();
+    var global_notification = Y.get('.global-notification');
+    if (global_notification.hasClass('hidden')) {
+        global_notification.addClass('transparent');
+        global_notification.removeClass('hidden');
+
+        var fade_in = new Y.Anim({
+            node: global_notification,
+            to: {opacity: 1},
+            duration: 0.3
+        });
+        var body_space = new Y.Anim({
+            node: 'body',
+            to: {'paddingTop': '40px'},
+            duration: 0.2,
+            easing: Y.Easing.easeOut
+        });
+        var black_link_space = new Y.Anim({
+            node: '.black-link',
+            to: {'top': '45px'},
+            duration: 0.2,
+            easing: Y.Easing.easeOut
+        });
+
+        fade_in.run();
+        body_space.run();
+        black_link_space.run();
+    }
+};
+
+Y.on('domready', function() {
+    var body = Y.get('body');
+    if (body.hasClass('private')) {
+        setup_privacy_notification();
+        display_privacy_notification();
+    }
+});

=== modified file 'loggerhead/templates/macros.pt'
--- loggerhead/templates/macros.pt	2011-03-03 10:58:57 +0000
+++ loggerhead/templates/macros.pt	2011-11-04 18:16:32 +0000
@@ -43,7 +43,7 @@
     <metal:block metal:define-slot="header_extras" />
   </head>
 
-<body>
+ <body tal:attributes="class python:branch.public_private_css()">
     <!-- Loggerhead Content Area -->
       
       <div metal:define-slot="backlink" />

=== modified file 'loggerhead/tests/test_controllers.py'
--- loggerhead/tests/test_controllers.py	2011-06-28 16:06:12 +0000
+++ loggerhead/tests/test_controllers.py	2011-11-02 01:56:02 +0000
@@ -1,11 +1,7 @@
-import simplejson
-
 from loggerhead.apps.branch import BranchWSGIApp
 from loggerhead.controllers.annotate_ui import AnnotateUI
 from loggerhead.controllers.inventory_ui import InventoryUI
-from loggerhead.controllers.revision_ui import RevisionUI
 from loggerhead.tests.test_simple import BasicTests, consume_app
-from loggerhead import util
 
 
 class TestInventoryUI(BasicTests):
@@ -86,6 +82,18 @@
         rev_ui.parse_args(env)
         self.assertIsInstance(rev_ui.get_values('', {}, []), dict)
 
+    def test_add_template_values(self):
+        branch_app = self.make_branch_app_for_revision_ui(
+                [('file', 'content\n')], [('file', 'new content\n')])
+        env = {'SCRIPT_NAME': '/',
+               'PATH_INFO': '/revision/1/non-existent-file',
+               'QUERY_STRING':'start_revid=1' }
+        revision_ui = branch_app.lookup_app(env)
+        path = revision_ui.parse_args(env)
+        values = revision_ui.get_values(path, revision_ui.kwargs, {})
+        revision_ui.add_template_values(values)
+        self.assertIs(values['diff_chunks'], None)
+
     def test_get_values_smoke(self):
         branch_app = self.make_branch_app_for_revision_ui(
                 [('file', 'content\n'), ('other-file', 'other\n')],
@@ -117,16 +125,16 @@
         tree = self.make_branch_and_tree('.')
         self.build_tree_contents([('filename', '')])
         tree.add(['filename'], [file_id])
-        for rev_id, text in rev_ids_texts:
+        for rev_id, text, message in rev_ids_texts:
             self.build_tree_contents([('filename', text)])
-            tree.commit(rev_id=rev_id, message='.')
+            tree.commit(rev_id=rev_id, message=message)
         tree.branch.lock_read()
         self.addCleanup(tree.branch.unlock)
         branch_app = BranchWSGIApp(tree.branch, friendly_name='test_name')
         return AnnotateUI(branch_app, branch_app.get_history)
 
     def test_annotate_file(self):
-        history = [('rev1', 'old\nold\n'), ('rev2', 'new\nold\n')]
+        history = [('rev1', 'old\nold\n', '.'), ('rev2', 'new\nold\n', '.')]
         ann_ui = self.make_annotate_ui_for_file_history('file_id', history)
         # A lot of this state is set up by __call__, but we'll do it directly
         # here.
@@ -138,6 +146,24 @@
         self.assertEqual('2', annotated[1].change.revno)
         self.assertEqual('1', annotated[2].change.revno)
 
+    def test_annotate_empty_comment(self):
+        # Testing empty comment handling without breaking
+        history = [('rev1', 'old\nold\n', '.'), ('rev2', 'new\nold\n', '')]
+        ann_ui = self.make_annotate_ui_for_file_history('file_id', history)
+        ann_ui.args = ['rev2']
+        annotate_info = ann_ui.get_values('filename',
+            kwargs={'file_id': 'file_id'}, headers={})
+
+    def test_annotate_file_zero_sized(self):
+        # Test against a zero-sized file without breaking. No annotation must be present.
+        history = [('rev1', '' , '.')]
+        ann_ui = self.make_annotate_ui_for_file_history('file_id', history)
+        ann_ui.args = ['rev1']
+        annotate_info = ann_ui.get_values('filename',
+            kwargs={'file_id': 'file_id'}, headers={})
+        annotated = annotate_info['annotated']
+        self.assertEqual(0, len(annotated))
+
 
 class TestFileDiffUI(BasicTests):
 
@@ -204,3 +230,31 @@
         revlog_ui = branch_app.lookup_app(env)
         self.assertOkJsonResponse(revlog_ui, env)
 
+
+class TestControllerHooks(BasicTests):
+
+    def test_dummy_hook(self):
+        return
+        # A hook that returns None doesn't influence the searching for
+        # a controller.
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/custom'}
+        myhook = lambda app, environ: None
+        branch = self.make_branch('.')
+        self.addCleanup(branch.lock_read().unlock)
+        app = self.make_branch_app(branch)
+        self.addCleanup(BranchWSGIApp.hooks.uninstall_named_hook, 'controller',
+            'captain hook')
+        BranchWSGIApp.hooks.install_named_hook('controller', myhook, "captain hook")
+        self.assertRaises(KeyError, app.lookup_app, env)
+
+    def test_working_hook(self):
+        # A hook can provide an app to use for a particular request.
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/custom'}
+        myhook = lambda app, environ: "I am hooked"
+        branch = self.make_branch('.')
+        self.addCleanup(branch.lock_read().unlock)
+        app = self.make_branch_app(branch)
+        self.addCleanup(BranchWSGIApp.hooks.uninstall_named_hook, 'controller',
+            'captain hook')
+        BranchWSGIApp.hooks.install_named_hook('controller', myhook, "captain hook")
+        self.assertEquals("I am hooked", app.lookup_app(env))

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2011-06-28 14:19:58 +0000
+++ loggerhead/tests/test_simple.py	2011-10-31 21:08:11 +0000
@@ -56,8 +56,8 @@
         self.assertEqual(None, start[2])
         simplejson.loads(content)
 
-    def make_branch_app(self, branch):
-        branch_app = BranchWSGIApp(branch, friendly_name='friendly-name')
+    def make_branch_app(self, branch, **kw):
+        branch_app = BranchWSGIApp(branch, friendly_name='friendly-name', **kw)
         branch_app._environ = {
             'wsgi.url_scheme':'',
             'SERVER_NAME':'',
@@ -84,6 +84,12 @@
         self.msg = 'a very exciting commit message <'
         self.revid = self.tree.commit(message=self.msg)
 
+    def test_public_private(self):
+        app = self.make_branch_app(self.tree.branch, private=True)
+        self.assertEqual(app.public_private_css(), 'private')
+        app = self.make_branch_app(self.tree.branch)
+        self.assertEqual(app.public_private_css(), 'public')
+
     def test_changes(self):
         app = self.setUpLoggerhead()
         res = app.get('/changes')

